import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Equendil.UI.Resources";
import "Equendil.UI.IconButton";
import "Equendil.UI.DecoratedContainer";
import "Equendil.UI.WindowManager";

local IconExpand = 0x41007E27; -- 16x16
local IconCollapse = 0x41007E26; -- 16x16
local HeaderBackground = 0x411105A6; -- 9x16

TypeColor = {
	["nil"] = {0.7, 0.7, 0.7},
	["boolean"] = {1, 0, 1}, 
	["number"] =  {1, 1, 0},
	["string"] = {0.4, 1, 0.4},
	["table"] = {0.4, 1, 1},
	["function"] = {0.4, 0.4, 1},
	["thread"] = {1, 0.4, 0.4},
	["userdata"] = {1, 1, 1},
};

local MetaTableColor = Turbine.UI.Color( 1, 0.7, 1, 1 );

local LuaNode = class( Turbine.UI.TreeNode );
function LuaNode:Constructor( key, value, hasCloseButton )
	Turbine.UI.TreeNode.Constructor( self );
	
	self.key = key;
	self.value = value;
	self.browsed = false;
	self.expanded = false;
	
	self.CloseRequested = Equendil.Utils.Event();
	
	self:SetSize( 545, 16 );

	self:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	self.labelKey = Turbine.UI.Label();
	self.labelKey:SetParent( self );
	self.labelKey:SetSize( 150, 16 );
	self.labelKey:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.labelKey:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.labelKey:SetMouseVisible( false );
	
	self.labelValue = Turbine.UI.Label();
	self.labelValue:SetParent( self );
	self.labelValue:SetSize( 385, 16 );
	self.labelValue:SetLeft( 160 );
	self.labelValue:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.labelValue:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.labelValue:SetMouseVisible( false );

	self.labelKey:SetForeColor( valueForeColor(self.key) );
	self.labelValue:SetForeColor( valueForeColor( self.value ) );

	if type( self.value ) == "table" then
		self.iconExpand = Turbine.UI.Control();
		self.iconExpand:SetParent( self );
		self.iconExpand:SetSize( 16, 16 );
		self.iconExpand:SetBackground( IconExpand );
		self.iconExpand:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
		self.iconExpand:SetMouseVisible( false );

		self:SetBackground( HeaderBackground );
		self.labelKey:SetLeft( 20 );
		self.labelValue:SetLeft( 170 );
		
		self.MouseDown = function( sender, args )
			if not self.browsed then
				self:Browse();
			end
			self.expanded = not self.expanded;
			if self.expanded then
				self.iconExpand:SetBackground( IconCollapse );
			else
				self.iconExpand:SetBackground( IconExpand );
			end
		end
	end

	self:SetMouseVisible( true );	
	
	if hasCloseButton then
		self.closeButton = Equendil.UI.ButtonCloseRed();
		self.closeButton:SetParent( self );
		self.closeButton:SetLeft( 529 );
		self.closeButton.Clicked = function() self.CloseRequested() end;
	end
	
	self:Update();
end

function LuaNode:Browse()
	local childList = self:GetChildNodes();
	local metatable = getmetatable( self.value );
	if metatable ~= nil then
		childList:Add( LuaNode( "__metatable__", metatable ) );
	end
	
	local sortedTable = {};
	for k, v in pairs( self.value ) do
		table.insert( sortedTable, k );
	end
	
	table.sort( sortedTable );
	
	for _, k in ipairs( sortedTable ) do
		local node = LuaNode( k, self.value[k] );
		childList:Add( node );
	end
	
	self.browsed = true;	
end

function LuaNode:Reset()
	local childNodes = self:GetChildNodes();
	childNodes:Clear();
	self:SetExpanded( false );
	self.iconExpand:SetBackground( IconExpand );
	
	self.browsed = false;
end

function valueBackColor( value )
	local color = TypeColor[type(value)];
	return Turbine.UI.Color( 0.4, color[1] * 0.5, color[2] * 0.5, color[3] * 0.5 );
end

function valueForeColor( value )
	if type(value) == "string" and value == "__metatable__" then
		return MetaTableColor;
	else
		local color = TypeColor[type(value)];
		return Turbine.UI.Color( 1, color[1], color[2], color[3] );
	end
end

function valueString( value )
	if type( value ) == "string" then
		if value == "__metatable__" then
			return "Metatable";
		else
			return '"' .. value .. '"';
		end
	elseif type( value ) == "nil" then
		return "Nil";
	else
		local success, s = pcall( tostring, value );
		if success then
			return s;
		else
			return "N/A";
		end
	end
end

function LuaNode:Update()
	if self.key == nil then 
		self.labelKey:SetText( "Root" );
	else
		self.labelKey:SetText( valueString( self.key ) );
	end
	self.labelValue:SetText( valueString( self.value ) );
end

Browser = class( Turbine.UI.Lotro.Window );
function Browser:Constructor( name )
	Turbine.UI.Lotro.Window.Constructor( self );
	
	self:SetSize( 575, 590 );
	self:SetPosition( 
		( Turbine.UI.Display:GetWidth() - self:GetWidth() ) / 2, 
		( Turbine.UI.Display:GetHeight() - self:GetHeight() ) / 2
	);	
	self:SetText( name );
	self:SetParent( nil );
	Equendil.UI.WindowManager:ManageWindow( self ); -- makes this window managed for HUD toggle / ESC

	self.background = Equendil.UI.DecoratedContainer( Equendil.UI.Decoration.GrainyBlue );
	self.background:SetParent( self );
	self.background:SetSize( 512, 550 );
	self.background:SetPosition( 31, 30 );
	
	self.scrollbar = Turbine.UI.Lotro.ScrollBar();
    self.scrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    self.scrollbar:SetParent( self );
    self.scrollbar:SetPosition( 555, 40 );
    self.scrollbar:SetSize( 10, 500 );
	
	self.luaTreeView = Turbine.UI.TreeView();
	self.luaTreeView:SetParent( self );
	self.luaTreeView:SetSize( 545, 500 );
	self.luaTreeView:SetPosition( 10, 40 );
	self.luaTreeView:SetIndentationWidth( 15 );
	self.luaTreeView:SetVerticalScrollBar( self.scrollbar );

	self.buttonRefresh = Turbine.UI.Lotro.Button();
	self.buttonRefresh:SetParent( self );
	self.buttonRefresh:SetSize( 150, 20 );
	self.buttonRefresh:SetPosition( 212, 550 );
	self.buttonRefresh:SetText( "Refresh all" );
	self.buttonRefresh.Click = function() 
		self:Refresh();
	end
	
	self.rootNodesList = self.luaTreeView:GetNodes();
end

function Browser:Add( key, value, expand, hasCloseButton )
	local rootNode = LuaNode( key, value, hasCloseButton );
	self.rootNodesList:Add( rootNode );
	if expand then
		rootNode:Browse();
		rootNode:SetExpanded( true );
	end
	
	if hasCloseButton then
		rootNode.CloseRequested = function()
			self.rootNodesList:Remove( rootNode );
		end
	end
end

function Browser:Refresh()
	local rootNodesCount = self.rootNodesList:GetCount();
	for i = 1, rootNodesCount do
		local rootNode = self.rootNodesList:Get( i );
		rootNode:Reset();
	end
end

